@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: audio.h 1.2 2002/11/03 11:50:02 kls Exp $
+ * $Id: audio.h 1.3 2005/02/12 12:20:19 kls Exp $
  */
 
 #ifndef __AUDIO_H
@@ -18,21 +18,22 @@ protected:
   cAudio(void);
 public:
   virtual ~cAudio();
-  virtual void Play(const uchar *Data, int Length) = 0;
-       // Plays the given block of audio Data. Must return as soon as possible.
-       // If the entire block of data can't be processed immediately, it must
-       // be copied and processed in a separate thread. The Data is always a
-       // complete PES audio packet.
+  virtual void Play(const uchar *Data, int Length, uchar Id) = 0;
+       ///< Plays the given block of audio Data. Must return as soon as possible.
+       ///< If the entire block of data can't be processed immediately, it must
+       ///< be copied and processed in a separate thread. The Data is always a
+       ///< complete PES audio packet. Id indicates the type of audio data this
+       ///< packet holds.
   virtual void Mute(bool On) = 0;
-       // Immediately sets the audio device to be silent (On==true) or to
-       // normal replay (On==false).
+       ///< Immediately sets the audio device to be silent (On==true) or to
+       ///< normal replay (On==false).
   virtual void Clear(void) = 0;
-       // Clears all data that might still be awaiting processing.
+       ///< Clears all data that might still be awaiting processing.
   };
 
 class cAudios : public cList<cAudio> {
 public:
-  void PlayAudio(const uchar *Data, int Length);
+  void PlayAudio(const uchar *Data, int Length, uchar Id);
   void MuteAudio(bool On);
   void ClearAudio(void);
   };
@@ -47,7 +48,7 @@ private:
 public:
   cExternalAudio(const char *Command);
   virtual ~cExternalAudio();
-  virtual void Play(const uchar *Data, int Length);
+  virtual void Play(const uchar *Data, int Length, uchar Id);
   virtual void Mute(bool On);
   virtual void Clear(void);
   };
