@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: device.h 1.79 2006/06/15 09:32:48 kls Exp $
+ * $Id: device.h 1.91 2008/02/23 13:13:04 kls Exp $
  */
 
 #ifndef __DEVICE_H
@@ -12,6 +12,7 @@
 
 #include "channels.h"
 #include "ci.h"
+#include "dvbsubtitle.h"
 #include "eit.h"
 #include "filter.h"
 #include "nit.h"
@@ -70,16 +71,15 @@ enum eTrackType { ttNone,
                   ttDolby,
                   ttDolbyFirst = ttDolby,
                   ttDolbyLast  = ttDolbyFirst + 15, // MAXDPIDS - 1
-                  /* future...
                   ttSubtitle,
                   ttSubtitleFirst = ttSubtitle,
-                  ttSubtitleLast  = ttSubtitleFirst + 7, // MAXSPIDS - 1
-                  */
+                  ttSubtitleLast  = ttSubtitleFirst + 31, // MAXSPIDS - 1
                   ttMaxTrackTypes
                 };
 
 #define IS_AUDIO_TRACK(t) (ttAudioFirst <= (t) && (t) <= ttAudioLast)
 #define IS_DOLBY_TRACK(t) (ttDolbyFirst <= (t) && (t) <= ttDolbyLast)
+#define IS_SUBTITLE_TRACK(t) (ttSubtitleFirst <= (t) && (t) <= ttSubtitleLast)
 
 struct tTrackId {
   uint16_t id;                  // The PES packet id or the PID.
@@ -90,15 +90,18 @@ struct tTrackId {
 class cPlayer;
 class cReceiver;
 class cPesAssembler;
+class cLiveSubtitle;
 
 /// The cDevice class is the base from which actual devices can be derived.
 
 class cDevice : public cThread {
+  friend class cLiveSubtitle;
 private:
   static int numDevices;
   static int useDevice;
   static cDevice *device[MAXDEVICES];
   static cDevice *primaryDevice;
+  static cDevice *avoidDevice;
 public:
   static int NumDevices(void) { return numDevices; }
          ///< Returns the total number of devices.
@@ -128,12 +131,24 @@ public:
          ///< Gets the device with the given Index.
          ///< \param Index must be in the range 0..numDevices-1.
          ///< \return A pointer to the device, or NULL if the Index was invalid.
-  static cDevice *GetDevice(const cChannel *Channel, int Priority = -1, bool *NeedsDetachReceivers = NULL);
+  static cDevice *GetDevice(const cChannel *Channel, int Priority, bool LiveView);
          ///< Returns a device that is able to receive the given Channel at the
          ///< given Priority, with the least impact on active recordings and
-         ///< live viewing.
-         ///< See ProvidesChannel() for more information on how
-         ///< priorities are handled, and the meaning of NeedsDetachReceivers.
+         ///< live viewing. The LiveView parameter tells whether the device will
+         ///< be used for live viewing or a recording.
+         ///< If the Channel is encrypted, a CAM slot that claims to be able to
+         ///< decrypt the channel is automatically selected and assigned to the
+         ///< returned device. Whether or not this combination of device and CAM
+         ///< slot is actually able to decrypt the channel can only be determined
+         ///< by checking the "scrambling control" bits of the received TS packets.
+         ///< The Action() function automatically does this and takes care that
+         ///< after detaching any receivers because the channel can't be decrypted,
+         ///< this device/CAM combination will be skipped in the next call to
+         ///< GetDevice().
+         ///< See also ProvidesChannel().
+  static void SetAvoidDevice(cDevice *Device) { avoidDevice = Device; }
+         ///< Sets the given Device to be temporarily avoided in the next call to
+         ///< GetDevice(const cChannel, int, bool).
   static void Shutdown(void);
          ///< Closes down all devices.
          ///< Must be called at the end of the program.
@@ -171,21 +186,14 @@ public:
          ///< Returns the card index of this device (0 ... MAXDEVICES - 1).
   int DeviceNumber(void) const;
          ///< Returns the number of this device (0 ... numDevices).
-  virtual int ProvidesCa(const cChannel *Channel) const;
-         ///< Checks whether this device provides the conditional access
-         ///< facilities to decrypt the given Channel.
-         ///< Returns 0 if the Channel can't be decrypted, 1 if this is a
-         ///< Free To Air channel or only exactly this device can decrypt it,
-         ///< and > 1 if this device can decrypt the Channel.
-         ///< If the result is greater than 1 and the device has more than one
-         ///< CAM, the value will be increased by the number of CAMs, which
-         ///< allows to select the device with the smallest number of CAMs
-         ///< in order to preserve resources for other recordings.
   virtual bool HasDecoder(void) const;
          ///< Tells whether this device has an MPEG decoder.
 
 // SPU facilities
 
+private:
+  cLiveSubtitle *liveSubtitle;
+  cDvbSubtitleConverter *dvbSubtitleConverter;
 public:
   virtual cSpuDecoder *GetSpuDecoder(void);
          ///< Returns a pointer to the device's SPU decoder (or NULL, if this
@@ -199,7 +207,9 @@ public:
   virtual bool ProvidesSource(int Source) const;
          ///< Returns true if this device can provide the given source.
   virtual bool ProvidesTransponder(const cChannel *Channel) const;
-         ///< XXX -> PLUGINS.html!
+         ///< Returns true if this device can provide the transponder of the
+         ///< given Channel (which implies that it can provide the Channel's
+         ///< source).
   virtual bool ProvidesTransponderExclusively(const cChannel *Channel) const;
          ///< Returns true if this is the only device that is able to provide
          ///< the given channel's transponder.
@@ -246,7 +256,7 @@ public:
          ///< channel number while replaying.
   void ForceTransferMode(void);
          ///< Forces the device into transfermode for the current channel.
-  virtual bool HasLock(int TimeoutMs = 0);//XXX PLUGINS.html
+  virtual bool HasLock(int TimeoutMs = 0);
          ///< Returns true if the device has a lock on the requested transponder.
          ///< Default is true, a specific device implementation may return false
          ///< to indicate that it is not ready yet.
@@ -296,12 +306,22 @@ private:
 protected:
   void StartSectionHandler(void);
        ///< A derived device that provides section data must call
-       ///< this function to actually set up the section handler.
+       ///< this function (typically in its constructor) to actually set
+       ///< up the section handler.
+  void StopSectionHandler(void);
+       ///< A device that has called StartSectionHandler() must call this
+       ///< function (typically in its destructor) to stop the section
+       ///< handler.
 public:
   virtual int OpenFilter(u_short Pid, u_char Tid, u_char Mask);
        ///< Opens a file handle for the given filter data.
        ///< A derived device that provides section data must
        ///< implement this function.
+  virtual void CloseFilter(int Handle);
+       ///< Closes a file handle that has previously been opened
+       ///< by OpenFilter(). If this is as simple as calling close(Handle),
+       ///< a derived class need not implement this function, because this
+       ///< is done by the default implementation.
   void AttachFilter(cFilter *Filter);
        ///< Attaches the given filter to this device.
   void Detach(cFilter *Filter);
@@ -309,10 +329,17 @@ public:
 
 // Common Interface facilities:
 
-protected:
-  cCiHandler *ciHandler;
+private:
+  time_t startScrambleDetection;
+  cCamSlot *camSlot;
 public:
-  cCiHandler *CiHandler(void) { return ciHandler; }
+  virtual bool HasCi(void);
+         ///< Returns true if this device has a Common Interface.
+  void SetCamSlot(cCamSlot *CamSlot);
+         ///< Sets the given CamSlot to be used with this device.
+  cCamSlot *CamSlot(void) const { return camSlot; }
+         ///< Returns the CAM slot that is currently used with this device,
+         ///< or NULL if no CAM slot is in use.
 
 // Image Grab facilities
 
@@ -354,12 +381,17 @@ public:
 private:
   tTrackId availableTracks[ttMaxTrackTypes];
   eTrackType currentAudioTrack;
+  eTrackType currentSubtitleTrack;
   cMutex mutexCurrentAudioTrack;
+  cMutex mutexCurrentSubtitleTrack;
   int currentAudioTrackMissingCount;
-  bool pre_1_3_19_PrivateStream;
+  bool autoSelectPreferredSubtitleLanguage;
+  int pre_1_3_19_PrivateStream;
 protected:
   virtual void SetAudioTrackDevice(eTrackType Type);
        ///< Sets the current audio track to the given value.
+  virtual void SetSubtitleTrackDevice(eTrackType Type);
+       ///< Sets the current subtitle track to the given value.
 public:
   void ClrAvailableTracks(bool DescriptionsOnly = false, bool IdsOnly = false);
        ///< Clears the list of currently availabe tracks. If DescriptionsOnly
@@ -376,18 +408,33 @@ public:
   const tTrackId *GetTrack(eTrackType Type);
        ///< Returns a pointer to the given track id, or NULL if Type is not
        ///< less than ttMaxTrackTypes.
+  int NumTracks(eTrackType FirstTrack, eTrackType LastTrack) const;
+       ///< Returns the number of tracks in the given range that are currently
+       ///< available.
   int NumAudioTracks(void) const;
        ///< Returns the number of audio tracks that are currently available.
        ///< This is just for information, to quickly find out whether there
        ///< is more than one audio track.
+  int NumSubtitleTracks(void) const;
+       ///< Returns the number of subtitle tracks that are currently available.
   eTrackType GetCurrentAudioTrack(void) { return currentAudioTrack; }
   bool SetCurrentAudioTrack(eTrackType Type);
        ///< Sets the current audio track to the given Type.
        ///< \return Returns true if Type is a valid audio track, false otherwise.
+  eTrackType GetCurrentSubtitleTrack(void) { return currentSubtitleTrack; }
+  bool SetCurrentSubtitleTrack(eTrackType Type, bool Manual = false);
+       ///< Sets the current subtitle track to the given Type.
+       ///< IF Manual is true, no automatic preferred subtitle language selection
+       ///< will be done for the rest of the current replay session, or until
+       ///< the channel is changed.
+       ///< \return Returns true if Type is a valid subtitle track, false otherwise.
   void EnsureAudioTrack(bool Force = false);
        ///< Makes sure an audio track is selected that is actually available.
        ///< If Force is true, the language and Dolby Digital settings will
        ///< be verified even if the current audio track is available.
+  void EnsureSubtitleTrack(void);
+       ///< Makes sure one of the preferred language subtitle tracks is selected.
+       ///< Only has an effect if Setup.DisplaySubtitles is on.
 
 // Audio facilities
 
@@ -446,6 +493,13 @@ protected:
        ///< Length) or not at all (returning 0 or -1 and setting 'errno' to EAGAIN).
        ///< \return Returns the number of bytes actually taken from Data, or -1
        ///< in case of an error.
+  virtual int PlaySubtitle(const uchar *Data, int Length);
+       ///< Plays the given data block as a subtitle.
+       ///< Data points to exactly one complete PES packet of the given Length.
+       ///< PlaySubtitle() shall process the packet either as a whole (returning
+       ///< Length) or not at all (returning 0 or -1 and setting 'errno' to EAGAIN).
+       ///< \return Returns the number of bytes actually taken from Data, or -1
+       ///< in case of an error.
   virtual int PlayPesPacket(const uchar *Data, int Length, bool VideoOnly = false);
        ///< Plays the single PES packet in Data with the given Length.
        ///< If VideoOnly is true, only the video will be displayed,
@@ -456,10 +510,19 @@ public:
        ///< Gets the current System Time Counter, which can be used to
        ///< synchronize audio and video. If this device is unable to
        ///< provide the STC, -1 will be returned.
+  virtual bool HasIBPTrickSpeed(void) { return false; }
+       ///< Returns true if this device can handle all frames in 'fast forward'
+       ///< trick speeds.
   virtual void TrickSpeed(int Speed);
        ///< Sets the device into a mode where replay is done slower.
        ///< Every single frame shall then be displayed the given number of
        ///< times.
+       ///< The cDvbPlayer uses the following values for the various speeds:
+       ///<                   1x   2x   3x
+       ///< Fast Forward       6    3    1
+       ///< Fast Reverse       6    3    1
+       ///< Slow Forward       8    4    2
+       ///< Slow Reverse      63   48   24
   virtual void Clear(void);
        ///< Clears all video and audio data from the device.
        ///< A derived class must call the base class function to make sure
@@ -512,11 +575,12 @@ public:
 private:
   cMutex mutexReceiver;
   cReceiver *receiver[MAXRECEIVERS];
-protected:
+public:
   int Priority(void) const;
       ///< Returns the priority of the current receiving session (0..MAXPRIORITY),
       ///< or -1 if no receiver is currently active. The primary device will
       ///< always return at least Setup.PrimaryLimit-1.
+protected:
   virtual bool OpenDvr(void);
       ///< Opens the DVR of this device and prepares it to deliver a Transport
       ///< Stream for use in a cReceiver.
@@ -530,8 +594,6 @@ protected:
       ///< false in case of a non recoverable error, otherwise it returns true,
       ///< even if Data is NULL.
 public:
-  int  Ca(void) const;
-       ///< Returns the ca of the current receiving session(s).
   bool Receiving(bool CheckAny = false) const;
        ///< Returns true if we are currently receiving.
   bool AttachReceiver(cReceiver *Receiver);
