@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: diseqc.c 1.4 2005/01/09 13:05:11 kls Exp $
+ * $Id: diseqc.c 1.6 2008/02/10 14:09:27 kls Exp $
  */
 
 #include "diseqc.h"
@@ -12,7 +12,7 @@
 #include "sources.h"
 #include "thread.h"
 
-// -- cDiseqc ----------------------------------------------------------------
+// --- cDiseqc ---------------------------------------------------------------
 
 cDiseqc::cDiseqc(void)
 {
@@ -65,7 +65,7 @@ char *cDiseqc::Wait(char *s)
         cCondWait::SleepMs(n);
      return p;
      }
-  esyslog("ERROR: illegal value for wait time in '%s'", s - 1);
+  esyslog("ERROR: invalid value for wait time in '%s'", s - 1);
   return NULL;
 }
 
@@ -85,7 +85,7 @@ char *cDiseqc::Codes(char *s)
                  t = skipspace(p);
                  }
               else {
-                 esyslog("ERROR: illegal code at '%s'", t);
+                 esyslog("ERROR: invalid code at '%s'", t);
                  return NULL;
                  }
               }
@@ -122,7 +122,7 @@ cDiseqc::eDiseqcActions cDiseqc::Execute(char **CurrentAction)
   return daNone;
 }
 
-// -- cDiseqcs ---------------------------------------------------------------
+// --- cDiseqcs --------------------------------------------------------------
 
 cDiseqcs Diseqcs;
 
