@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: interface.c 1.75 2006/03/31 14:20:04 kls Exp $
+ * $Id: interface.c 1.77 2008/02/10 15:49:15 kls Exp $
  */
 
 #include "interface.h"
@@ -86,12 +86,7 @@ bool cInterface::QueryKeys(cRemote *Remote, cSkinDisplayMenu *DisplayMenu)
      DisplayMenu->SetItem(tr("Phase 2: Learning specific key codes"), 2, false, false);
      eKeys NewKey = kUp;
      while (NewKey != kNone) {
-           char *Prompt;
-           char buf[32];
-           snprintf(buf, sizeof(buf), "Key$%s", cKey::ToString(NewKey));
-           asprintf(&Prompt, tr("Press key for '%s'"), tr(buf));
-           DisplayMenu->SetItem(Prompt, 4, false, false);
-           free(Prompt);
+           DisplayMenu->SetItem(cString::sprintf(tr("Press key for '%s'"), cKey::ToString(NewKey, true)), 4, false, false);
            cRemote::Clear();
            DisplayMenu->Flush();
            for (eKeys k = NewKey; k == NewKey; ) {
