@@ -4,11 +4,12 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: osd.c 1.68 2007/02/17 16:05:52 kls Exp $
+ * $Id: osd.c 1.75 2007/10/12 12:38:36 kls Exp $
  */
 
 #include "osd.h"
 #include <math.h>
+#include <stdarg.h>
 #include <stdlib.h>
 #include <sys/ioctl.h>
 #include <sys/stat.h>
@@ -20,6 +21,18 @@
 cPalette::cPalette(int Bpp)
 {
   SetBpp(Bpp);
+  SetAntiAliasGranularity(10, 10);
+}
+
+void cPalette::SetAntiAliasGranularity(uint FixedColors, uint BlendColors)
+{
+  if (FixedColors >= MAXNUMCOLORS || BlendColors == 0)
+     antiAliasGranularity = MAXNUMCOLORS - 1;
+  else {
+     int ColorsForBlending = MAXNUMCOLORS - FixedColors;
+     int ColorsPerBlend = ColorsForBlending / BlendColors + 2; // +2 = the full foreground and background colors, which are amoung the fixed colors
+     antiAliasGranularity = double(MAXNUMCOLORS - 1) / (ColorsPerBlend - 1);
+     }
 }
 
 void cPalette::Reset(void)
@@ -30,18 +43,23 @@ void cPalette::Reset(void)
 
 int cPalette::Index(tColor Color)
 {
+  // Check if color is already defined:
   for (int i = 0; i < numColors; i++) {
       if (color[i] == Color)
          return i;
       }
+  // No exact color, try a close one:
+  int i = ClosestColor(Color, 4);
+  if (i >= 0)
+     return i;
+  // No close one, try to define a new one:
   if (numColors < maxColors) {
      color[numColors++] = Color;
      modified = true;
      return numColors - 1;
      }
-  dsyslog("too many different colors used in palette");
-  //TODO: return the index of the "closest" color?
-  return 0;
+  // Out of colors, so any close color must do:
+  return ClosestColor(Color);
 }
 
 void cPalette::SetBpp(int Bpp)
@@ -64,7 +82,7 @@ void cPalette::SetColor(int Index, tColor Color)
      }
 }
 
-const tColor *cPalette::Colors(int &NumColors)
+const tColor *cPalette::Colors(int &NumColors) const
 {
   NumColors = numColors;
   return numColors ? color : NULL;
@@ -91,6 +109,48 @@ void cPalette::Replace(const cPalette &Palette)
   for (int i = 0; i < Palette.numColors; i++)
       SetColor(i, Palette.color[i]);
   numColors = Palette.numColors;
+  antiAliasGranularity = Palette.antiAliasGranularity;
+}
+
+tColor cPalette::Blend(tColor ColorFg, tColor ColorBg, uint8_t Level) const
+{
+  if (antiAliasGranularity > 0)
+     Level = uint8_t(int(Level / antiAliasGranularity + 0.5) * antiAliasGranularity);
+  int Af = (ColorFg & 0xFF000000) >> 24;
+  int Rf = (ColorFg & 0x00FF0000) >> 16;
+  int Gf = (ColorFg & 0x0000FF00) >>  8;
+  int Bf = (ColorFg & 0x000000FF);
+  int Ab = (ColorBg & 0xFF000000) >> 24;
+  int Rb = (ColorBg & 0x00FF0000) >> 16;
+  int Gb = (ColorBg & 0x0000FF00) >>  8;
+  int Bb = (ColorBg & 0x000000FF);
+  int A = (Ab + (Af - Ab) * Level / 0xFF) & 0xFF;
+  int R = (Rb + (Rf - Rb) * Level / 0xFF) & 0xFF;
+  int G = (Gb + (Gf - Gb) * Level / 0xFF) & 0xFF;
+  int B = (Bb + (Bf - Bb) * Level / 0xFF) & 0xFF;
+  return (A << 24) | (R << 16) | (G << 8) | B;
+}
+
+int cPalette::ClosestColor(tColor Color, int MaxDiff) const
+{
+  int n = 0;
+  int d = INT_MAX;
+  int A1 = (Color & 0xFF000000) >> 24;
+  int R1 = (Color & 0x00FF0000) >> 16;
+  int G1 = (Color & 0x0000FF00) >>  8;
+  int B1 = (Color & 0x000000FF);
+  for (int i = 0; i < numColors; i++) {
+      int A2 = (color[i] & 0xFF000000) >> 24;
+      int R2 = (color[i] & 0x00FF0000) >> 16;
+      int G2 = (color[i] & 0x0000FF00) >>  8;
+      int B2 = (color[i] & 0x000000FF);
+      int diff = (abs(A1 - A2) << 1) + (abs(R1 - R2) << 1) + (abs(G1 - G2) << 1) + (abs(B1 - B2) << 1);
+      if (diff < d) {
+         d = diff;
+         n = i;
+         }
+      }
+  return d <= MaxDiff ? n : -1;
 }
 
 // --- cBitmap ---------------------------------------------------------------
@@ -261,6 +321,8 @@ bool cBitmap::LoadXpm(const char *FileName)
 
 bool cBitmap::SetXpm(const char *const Xpm[], bool IgnoreNone)
 {
+  if (!Xpm)
+     return false;
   const char *const *p = Xpm;
   int w, h, n, c;
   if (4 != sscanf(*p, "%d %d %d %d", &w, &h, &n, &c)) {
@@ -387,13 +449,13 @@ void cBitmap::DrawText(int x, int y, const char *s, tColor ColorFg, tColor Color
      int w = Font->Width(s);
      int h = Font->Height();
      int limit = 0;
+     int cw = Width ? Width : w;
+     int ch = Height ? Height : h;
+     if (!Intersects(x, y, x + cw - 1, y + ch - 1))
+        return;
+     if (ColorBg != clrTransparent)
+        DrawRectangle(x, y, x + cw - 1, y + ch - 1, ColorBg);
      if (Width || Height) {
-        int cw = Width ? Width : w;
-        int ch = Height ? Height : h;
-        if (!Intersects(x, y, x + cw - 1, y + ch - 1))
-           return;
-        if (ColorBg != clrTransparent)
-           DrawRectangle(x, y, x + cw - 1, y + ch - 1, ColorBg);
         limit = x + cw - x0;
         if (Width) {
            if ((Alignment & taLeft) != 0)
@@ -420,30 +482,9 @@ void cBitmap::DrawText(int x, int y, const char *s, tColor ColorFg, tColor Color
               }
            }
         }
-     else if (!Intersects(x, y, x + w - 1, y + h - 1))
-        return;
      x -= x0;
      y -= y0;
-     tIndex fg = Index(ColorFg);
-     tIndex bg = (ColorBg != clrTransparent) ? Index(ColorBg) : 0;
-     while (s && *s) {
-           const cFont::tCharData *CharData = Font->CharData(*s++);
-           if (limit && int(x + CharData->width) > limit)
-              break; // we don't draw partial characters
-           if (int(x + CharData->width) > 0) {
-              for (int row = 0; row < h; row++) {
-                  cFont::tPixelData PixelData = CharData->lines[row];
-                  for (int col = CharData->width; col-- > 0; ) {
-                      if (ColorBg != clrTransparent || (PixelData & 1))
-                         SetIndex(x + col, y + row, (PixelData & 1) ? fg : bg);
-                      PixelData >>= 1;
-                      }
-                  }
-              }
-           x += CharData->width;
-           if (x > width - 1)
-              break;
-           }
+     Font->DrawText(this, x, y, s, ColorFg, ColorBg, limit);
      }
 }
 
@@ -599,20 +640,103 @@ const tIndex *cBitmap::Data(int x, int y)
   return &bitmap[y * width + x];
 }
 
+void cBitmap::ReduceBpp(const cPalette &Palette)
+{
+  int NewBpp = Palette.Bpp();
+  if (Bpp() == 4 && NewBpp == 2) {
+     for (int i = width * height; i--; ) {
+         tIndex p = bitmap[i];
+         bitmap[i] = (p >> 2) | ((p & 0x03) != 0);
+         }
+     }
+  else if (Bpp() == 8) {
+     if (NewBpp == 2) {
+        for (int i = width * height; i--; ) {
+            tIndex p = bitmap[i];
+            bitmap[i] = (p >> 6) | ((p & 0x30) != 0);
+            }
+        }
+     else if (NewBpp == 4) {
+        for (int i = width * height; i--; ) {
+            tIndex p = bitmap[i];
+            bitmap[i] = p >> 4;
+            }
+        }
+     else
+        return;
+     }
+  else
+     return;
+  SetBpp(NewBpp);
+  Replace(Palette);
+}
+
+void cBitmap::ShrinkBpp(int NewBpp)
+{
+  int NumOldColors;
+  const tColor *Colors = this->Colors(NumOldColors);
+  if (Colors) {
+     // Find the most frequently used colors and create a map table:
+     int Used[MAXNUMCOLORS] = { 0 };
+     int Map[MAXNUMCOLORS] = { 0 };
+     for (int i = width * height; i--; )
+         Used[bitmap[i]]++;
+     int MaxNewColors = (NewBpp == 4) ? 16 : 4;
+     cPalette NewPalette(NewBpp);
+     for (int i = 0; i < MaxNewColors; i++) {
+         int Max = 0;
+         int Index = -1;
+         for (int n = 0; n < NumOldColors; n++) {
+             if (Used[n] > Max) {
+                Max = Used[n];
+                Index = n;
+                }
+             }
+         if (Index >= 0) {
+            Used[Index] = 0;
+            Map[Index] = i;
+            NewPalette.SetColor(i, Colors[Index]);
+            }
+         else
+            break;
+         }
+     // Complete the map table for all other colors (will be set to closest match):
+     for (int n = 0; n < NumOldColors; n++) {
+         if (Used[n])
+            Map[n] = NewPalette.Index(Colors[n]);
+         }
+     // Do the actual index mapping:
+     for (int i = width * height; i--; )
+         bitmap[i] = Map[bitmap[i]];
+     SetBpp(NewBpp);
+     Replace(NewPalette);
+     }
+}
+
 // --- cOsd ------------------------------------------------------------------
 
-int cOsd::isOpen = 0;
+int cOsd::osdLeft = 0;
+int cOsd::osdTop = 0;
+int cOsd::osdWidth = 0;
+int cOsd::osdHeight = 0;
+cVector<cOsd *> cOsd::Osds;
 
-cOsd::cOsd(int Left, int Top)
+cOsd::cOsd(int Left, int Top, uint Level)
 {
-  if (isOpen)
-     esyslog("ERROR: OSD opened without closing previous OSD!");
   savedRegion = NULL;
   numBitmaps = 0;
   left = Left;
   top = Top;
   width = height = 0;
-  isOpen++;
+  level = Level;
+  active = false;
+  for (int i = 0; i < Osds.Size(); i++) {
+      if (Osds[i]->level > level) {
+         Osds.Insert(this, i);
+         return;
+         }
+      }
+  Osds.Append(this);
 }
 
 cOsd::~cOsd()
@@ -620,7 +744,28 @@ cOsd::~cOsd()
   for (int i = 0; i < numBitmaps; i++)
       delete bitmaps[i];
   delete savedRegion;
-  isOpen--;
+  for (int i = 0; i < Osds.Size(); i++) {
+      if (Osds[i] == this) {
+         Osds.Remove(i);
+         if (Osds.Size())
+            Osds[0]->SetActive(true);
+         break;
+         }
+      }
+}
+
+void cOsd::SetOsdPosition(int Left, int Top, int Width, int Height)
+{
+  osdLeft = Left;
+  osdTop = Top;
+  osdWidth = min(max(Width, MINOSDWIDTH), MAXOSDWIDTH);
+  osdHeight = min(max(Height, MINOSDHEIGHT), MAXOSDHEIGHT);
+}
+
+void cOsd::SetAntiAliasGranularity(uint FixedColors, uint BlendColors)
+{
+  for (int i = 0; i < numBitmaps; i++)
+      bitmaps[i]->SetAntiAliasGranularity(FixedColors, BlendColors);
 }
 
 cBitmap *cOsd::GetBitmap(int Area)
@@ -648,19 +793,18 @@ eOsdError cOsd::CanHandleAreas(const tArea *Areas, int NumAreas)
 
 eOsdError cOsd::SetAreas(const tArea *Areas, int NumAreas)
 {
-  eOsdError Result = oeUnknown;
-  if (numBitmaps == 0) {
-     Result = CanHandleAreas(Areas, NumAreas);
-     if (Result == oeOk) {
-        width = height = 0;
-        for (int i = 0; i < NumAreas; i++) {
-            bitmaps[numBitmaps++] = new cBitmap(Areas[i].Width(), Areas[i].Height(), Areas[i].bpp, Areas[i].x1, Areas[i].y1);
-            width = max(width, Areas[i].x2 + 1);
-            height = max(height, Areas[i].y2 + 1);
-            }
-        }
+  eOsdError Result = CanHandleAreas(Areas, NumAreas);
+  if (Result == oeOk) {
+     while (numBitmaps)
+           delete bitmaps[--numBitmaps];
+     width = height = 0;
+     for (int i = 0; i < NumAreas; i++) {
+         bitmaps[numBitmaps++] = new cBitmap(Areas[i].Width(), Areas[i].Height(), Areas[i].bpp, Areas[i].x1, Areas[i].y1);
+         width = max(width, Areas[i].x2 + 1);
+         height = max(height, Areas[i].y2 + 1);
+         }
      }
-  if (Result != oeOk)
+  else
      esyslog("ERROR: cOsd::SetAreas returned %d", Result);
   return Result;
 }
@@ -744,15 +888,23 @@ cOsdProvider::~cOsdProvider()
   osdProvider = NULL;
 }
 
-cOsd *cOsdProvider::NewOsd(int Left, int Top)
+cOsd *cOsdProvider::NewOsd(int Left, int Top, uint Level)
 {
-  if (cOsd::IsOpen())
+  if (Level == OSD_LEVEL_DEFAULT && cOsd::IsOpen())
      esyslog("ERROR: attempt to open OSD while it is already open - using dummy OSD!");
-  else if (osdProvider)
-     return osdProvider->CreateOsd(Left, Top);
+  else if (osdProvider) {
+     cOsd *ActiveOsd = cOsd::Osds.Size() ? cOsd::Osds[0] : NULL;
+     cOsd *Osd = osdProvider->CreateOsd(Left, Top, Level);
+     if (Osd == cOsd::Osds[0]) {
+        if (ActiveOsd)
+           ActiveOsd->SetActive(false);
+        Osd->SetActive(true);
+        }
+     return Osd;
+     }
   else
      esyslog("ERROR: no OSD provider available - using dummy OSD!");
-  return new cOsd(Left, Top); // create a dummy cOsd, so that access won't result in a segfault
+  return new cOsd(Left, Top, 999); // create a dummy cOsd, so that access won't result in a segfault
 }
 
 void cOsdProvider::Shutdown(void)
