@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: recorder.h 1.4 2005/08/13 11:31:18 kls Exp $
+ * $Id: recorder.h 1.5 2007/01/05 10:44:05 kls Exp $
  */
 
 #ifndef __RECORDER_H
@@ -28,9 +28,9 @@ protected:
   virtual void Receive(uchar *Data, int Length);
   virtual void Action(void);
 public:
-  cRecorder(const char *FileName, int Ca, int Priority, int VPid, const int *APids, const int *DPids, const int *SPids);
-               // Creates a new recorder that requires conditional access Ca, has
-               // the given Priority and will record the given PIDs into the file FileName.
+  cRecorder(const char *FileName, tChannelID ChannelID, int Priority, int VPid, const int *APids, const int *DPids, const int *SPids);
+               // Creates a new recorder for the channel with the given ChannelID and
+               // the given Priority that will record the given PIDs into the file FileName.
   virtual ~cRecorder();
   };
 
