@@ -4,10 +4,11 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: recording.c 1.151 2006/10/07 12:46:22 kls Exp $
+ * $Id: recording.c 1.162 2008/02/24 10:28:53 kls Exp $
  */
 
 #include "recording.h"
+#include <ctype.h>
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -80,6 +81,7 @@ cRemoveDeletedRecordingsThread::cRemoveDeletedRecordingsThread(void)
 
 void cRemoveDeletedRecordingsThread::Action(void)
 {
+  SetPriority(19);
   // Make sure only one instance of VDR does this:
   cLockFile LockFile(VideoDirectory);
   if (LockFile.Lock()) {
@@ -142,10 +144,12 @@ void AssertFreeDiskSpace(int Priority, bool Force)
         cThreadLock DeletedRecordingsLock(&DeletedRecordings);
         if (DeletedRecordings.Count()) {
            cRecording *r = DeletedRecordings.First();
-           cRecording *r0 = r;
+           cRecording *r0 = NULL;
            while (r) {
-                 if (r->start < r0->start)
-                    r0 = r;
+                 if (IsOnVideoDirectoryFileSystem(r->FileName())) { // only remove recordings that will actually increase the free video disk space
+                    if (!r0 || r->start < r0->start)
+                       r0 = r;
+                    }
                  r = DeletedRecordings.Next(r);
                  }
            if (r0 && r0->Remove()) {
@@ -154,11 +158,13 @@ void AssertFreeDiskSpace(int Priority, bool Force)
               return;
               }
            }
-        // DeletedRecordings was empty, so to be absolutely sure there are no
-        // deleted recordings we need to double check:
-        DeletedRecordings.Update(true);
-        if (DeletedRecordings.Count())
-           return; // the next call will actually remove it
+        else {
+           // DeletedRecordings was empty, so to be absolutely sure there are no
+           // deleted recordings we need to double check:
+           DeletedRecordings.Update(true);
+           if (DeletedRecordings.Count())
+              return; // the next call will actually remove it
+           }
         // No "deleted" files to remove, so let's see if we can delete a recording:
         isyslog("...no deleted recording found, trying to delete an old recording...");
         cThreadLock RecordingsLock(&Recordings);
@@ -166,15 +172,17 @@ void AssertFreeDiskSpace(int Priority, bool Force)
            cRecording *r = Recordings.First();
            cRecording *r0 = NULL;
            while (r) {
-                 if (!r->IsEdited() && r->lifetime < MAXLIFETIME) { // edited recordings and recordings with MAXLIFETIME live forever
-                    if ((r->lifetime == 0 && Priority > r->priority) || // the recording has no guaranteed lifetime and the new recording has higher priority
-                        (r->lifetime > 0 && (time(NULL) - r->start) / SECSINDAY >= r->lifetime)) { // the recording's guaranteed lifetime has expired
-                       if (r0) {
-                          if (r->priority < r0->priority || (r->priority == r0->priority && r->start < r0->start))
-                             r0 = r; // in any case we delete the one with the lowest priority (or the older one in case of equal priorities)
+                 if (IsOnVideoDirectoryFileSystem(r->FileName())) { // only delete recordings that will actually increase the free video disk space
+                    if (!r->IsEdited() && r->lifetime < MAXLIFETIME) { // edited recordings and recordings with MAXLIFETIME live forever
+                       if ((r->lifetime == 0 && Priority > r->priority) || // the recording has no guaranteed lifetime and the new recording has higher priority
+                           (r->lifetime > 0 && (time(NULL) - r->start) / SECSINDAY >= r->lifetime)) { // the recording's guaranteed lifetime has expired
+                          if (r0) {
+                             if (r->priority < r0->priority || (r->priority == r0->priority && r->start < r0->start))
+                                r0 = r; // in any case we delete the one with the lowest priority (or the older one in case of equal priorities)
+                             }
+                          else
+                             r0 = r;
                           }
-                       else
-                          r0 = r;
                        }
                     }
                  r = Recordings.Next(r);
@@ -262,6 +270,7 @@ void cResumeFile::Delete(void)
 cRecordingInfo::cRecordingInfo(const cChannel *Channel, const cEvent *Event)
 {
   channelID = Channel ? Channel->GetChannelID() : tChannelID::InvalidID;
+  channelName = Channel ? strdup(Channel->Name()) : NULL;
   ownEvent = Event ? NULL : new cEvent(0);
   event = ownEvent ? ownEvent : Event;
   aux = NULL;
@@ -295,6 +304,17 @@ cRecordingInfo::cRecordingInfo(const cChannel *Channel, const cEvent *Event)
                strn0cpy(Component->language, s, sizeof(Component->language));
             }
          }
+     // The same applies to subtitles:
+     for (int i = 0; i < MAXSPIDS; i++) {
+         const char *s = Channel->Slang(i);
+         if (*s) {
+            tComponent *Component = Components->GetComponent(i, 3, 3);
+            if (!Component)
+               Components->SetComponent(Components->NumComponents(), 3, 3, s, NULL);
+            else if (strlen(s) > strlen(Component->language))
+               strn0cpy(Component->language, s, sizeof(Component->language));
+            }
+         }
      if (Components != event->Components())
         ((cEvent *)event)->SetComponents(Components);
      }
@@ -304,6 +324,7 @@ cRecordingInfo::~cRecordingInfo()
 {
   delete ownEvent;
   free(aux);
+  free(channelName);
 }
 
 void cRecordingInfo::SetData(const char *Title, const char *ShortText, const char *Description)
@@ -334,8 +355,11 @@ bool cRecordingInfo::Read(FILE *f)
            switch (*s) {
              case 'C': {
                          char *p = strchr(t, ' ');
-                         if (p)
+                         if (p) {
+                            free(channelName);
+                            channelName = strdup(compactspace(p));
                             *p = 0; // strips optional channel name
+                            }
                          if (*t)
                             channelID = tChannelID::FromString(t);
                        }
@@ -375,7 +399,7 @@ bool cRecordingInfo::Read(FILE *f)
 bool cRecordingInfo::Write(FILE *f, const char *Prefix) const
 {
   if (channelID.Valid())
-     fprintf(f, "%sC %s\n", Prefix, *channelID.ToString());
+     fprintf(f, "%sC %s%s%s\n", Prefix, *channelID.ToString(), channelName ? " " : "", channelName ? channelName : "");
   event->Dump(f, Prefix, true);
   if (aux)
      fprintf(f, "%s@ %s\n", Prefix, aux);
@@ -406,34 +430,14 @@ char *ExchangeChars(char *s, bool ToFileSystem)
            // The VFAT file system can't handle all characters, so we
            // have to take extra efforts to encode/decode them:
            if (ToFileSystem) {
+              const char *InvalidChars = "\"\\/:*?|<>#";
               switch (*p) {
-                     // characters that can be used "as is":
-                     case '!':
-                     case '@':
-                     case '$':
-                     case '%':
-                     case '&':
-                     case '(':
-                     case ')':
-                     case '+':
-                     case ',':
-                     case '-':
-                     case ';':
-                     case '=':
-                     case '0' ... '9':
-                     case 'a' ... 'z':
-                     case 'A' ... 'Z':
-                     case '': case '':
-                     case '': case '':
-                     case '': case '':
-                     case '':
-                          break;
                      // characters that can be mapped to other characters:
                      case ' ': *p = '_'; break;
                      case '~': *p = '/'; break;
                      // characters that have to be encoded:
                      default:
-                       if (*p != '.' || !*(p + 1) || *(p + 1) == '~') { // Windows can't handle '.' at the end of directory names
+                       if (strchr(InvalidChars, *p) || *p == '.' && (!*(p + 1) || *(p + 1) == '~')) { // Windows can't handle '.' at the end of file/directory names
                           int l = p - s;
                           s = (char *)realloc(s, strlen(s) + 10);
                           p = s + l;
@@ -450,14 +454,16 @@ char *ExchangeChars(char *s, bool ToFileSystem)
                 // mapped characters:
                 case '_': *p = ' '; break;
                 case '/': *p = '~'; break;
-                // encodes characters:
+                // encoded characters:
                 case '#': {
-                     if (strlen(p) > 2) {
+                     if (strlen(p) > 2 && isxdigit(*(p + 1)) && isxdigit(*(p + 2))) {
                         char buf[3];
                         sprintf(buf, "%c%c", *(p + 1), *(p + 2));
                         unsigned char c = strtol(buf, NULL, 16);
-                        *p = c;
-                        memmove(p + 1, p + 3, strlen(p) - 2);
+                        if (c) {
+                           *p = c;
+                           memmove(p + 1, p + 3, strlen(p) - 2);
+                           }
                         }
                      }
                      break;
@@ -500,7 +506,7 @@ cRecording::cRecording(cTimer *Timer, const cEvent *Event)
      Subtitle = " ";
   else if (strlen(Subtitle) > MAX_SUBTITLE_LENGTH) {
      // let's make sure the Subtitle doesn't produce too long a file name:
-     strn0cpy(SubtitleBuffer, Subtitle, MAX_SUBTITLE_LENGTH);
+     Utf8Strn0Cpy(SubtitleBuffer, Subtitle, MAX_SUBTITLE_LENGTH);
      Subtitle = SubtitleBuffer;
      }
   char *macroTITLE   = strstr(Timer->File(), TIMERMACRO_TITLE);
@@ -525,7 +531,7 @@ cRecording::cRecording(cTimer *Timer, const cEvent *Event)
   else if (Timer->IsSingleEvent() || !Setup.UseSubtitle)
      name = strdup(Timer->File());
   else
-     asprintf(&name, "%s~%s", Timer->File(), Subtitle);
+     name = strdup(cString::sprintf("%s~%s", Timer->File(), Subtitle));
   // substitute characters that would cause problems in file names:
   strreplace(name, '\n', ' ');
   start = Timer->StartTime();
@@ -566,22 +572,19 @@ cRecording::cRecording(const char *FileName)
         }
      GetResume();
      // read an optional info file:
-     char *InfoFileName = NULL;
-     asprintf(&InfoFileName, "%s%s", fileName, INFOFILESUFFIX);
+     cString InfoFileName = cString::sprintf("%s%s", fileName, INFOFILESUFFIX);
      FILE *f = fopen(InfoFileName, "r");
      if (f) {
         if (!info->Read(f))
-           esyslog("ERROR: EPG data problem in file %s", InfoFileName);
+           esyslog("ERROR: EPG data problem in file %s", *InfoFileName);
         fclose(f);
         }
      else if (errno != ENOENT)
-        LOG_ERROR_STR(InfoFileName);
-     free(InfoFileName);
+        LOG_ERROR_STR(*InfoFileName);
 #ifdef SUMMARYFALLBACK
      // fall back to the old 'summary.vdr' if there was no 'info.vdr':
      if (isempty(info->Title())) {
-        char *SummaryFileName = NULL;
-        asprintf(&SummaryFileName, "%s%s", fileName, SUMMARYFILESUFFIX);
+        cString SummaryFileName = cString::sprintf("%s%s", fileName, SUMMARYFILESUFFIX);
         FILE *f = fopen(SummaryFileName, "r");
         if (f) {
            int line = 0;
@@ -627,8 +630,7 @@ cRecording::cRecording(const char *FileName)
                free(data[i]);
            }
         else if (errno != ENOENT)
-           LOG_ERROR_STR(SummaryFileName);
-        free(SummaryFileName);
+           LOG_ERROR_STR(*SummaryFileName);
         }
 #endif
      }
@@ -697,7 +699,7 @@ const char *cRecording::FileName(void) const
      struct tm tm_r;
      struct tm *t = localtime_r(&start, &tm_r);
      name = ExchangeChars(name, true);
-     asprintf(&fileName, NAMEFORMAT, VideoDirectory, name, t->tm_year + 1900, t->tm_mon + 1, t->tm_mday, t->tm_hour, t->tm_min, priority, lifetime);
+     fileName = strdup(cString::sprintf(NAMEFORMAT, VideoDirectory, name, t->tm_year + 1900, t->tm_mon + 1, t->tm_mday, t->tm_hour, t->tm_min, priority, lifetime));
      name = ExchangeChars(name, false);
      }
   return fileName;
@@ -716,7 +718,7 @@ const char *cRecording::Title(char Delimiter, bool NewIndicator, int Level) cons
         s++;
      else
         s = name;
-     asprintf(&titleBuffer, "%02d.%02d.%02d%c%02d:%02d%c%c%s",
+     titleBuffer = strdup(cString::sprintf("%02d.%02d.%02d%c%02d:%02d%c%c%s",
                             t->tm_mday,
                             t->tm_mon + 1,
                             t->tm_year % 100,
@@ -725,7 +727,7 @@ const char *cRecording::Title(char Delimiter, bool NewIndicator, int Level) cons
                             t->tm_min,
                             New,
                             Delimiter,
-                            s);
+                            s));
      // let's not display a trailing '~':
      if (!NewIndicator)
         stripspace(titleBuffer);
@@ -785,16 +787,14 @@ bool cRecording::IsEdited(void) const
 
 bool cRecording::WriteInfo(void)
 {
-  char *InfoFileName = NULL;
-  asprintf(&InfoFileName, "%s%s", fileName, INFOFILESUFFIX);
+  cString InfoFileName = cString::sprintf("%s%s", fileName, INFOFILESUFFIX);
   FILE *f = fopen(InfoFileName, "w");
   if (f) {
      info->Write(f);
      fclose(f);
      }
   else
-     LOG_ERROR_STR(InfoFileName);
-  free(InfoFileName);
+     LOG_ERROR_STR(*InfoFileName);
   return true;
 }
 
@@ -833,6 +833,32 @@ bool cRecording::Remove(void)
   return RemoveVideoFile(FileName());
 }
 
+bool cRecording::Undelete(void)
+{
+  bool result = true;
+  char *NewName = strdup(FileName());
+  char *ext = strrchr(NewName, '.');
+  if (ext && strcmp(ext, DELEXT) == 0) {
+     strncpy(ext, RECEXT, strlen(ext));
+     if (access(NewName, F_OK) == 0) {
+        // the new name already exists, so let's not remove that one:
+        esyslog("ERROR: attempt to undelete '%s', while recording '%s' exists", FileName(), NewName);
+        result = false;
+        }
+     else {
+        isyslog("undeleting recording '%s'", FileName());
+        if (access(FileName(), F_OK) == 0)
+           result = RenameVideoFile(FileName(), NewName);
+        else {
+           isyslog("deleted recording '%s' vanished", FileName());
+           result = false;
+           }
+        }
+     }
+  free(NewName);
+  return result;
+}
+
 void cRecording::ResetResume(void) const
 {
   resume = RESUME_NOT_INITIALIZED;
@@ -885,8 +911,7 @@ void cRecordings::ScanVideoDir(const char *DirName, bool Foreground, int LinkLev
   struct dirent *e;
   while ((Foreground || Running()) && (e = d.Next()) != NULL) {
         if (strcmp(e->d_name, ".") && strcmp(e->d_name, "..")) {
-           char *buffer;
-           asprintf(&buffer, "%s/%s", DirName, e->d_name);
+           char *buffer = strdup(AddDirectory(DirName, e->d_name));
            struct stat st;
            if (stat(buffer, &st) == 0) {
               int Link = 0;
@@ -968,10 +993,12 @@ bool cRecordings::Update(bool Wait)
 
 cRecording *cRecordings::GetByName(const char *FileName)
 {
-  for (cRecording *recording = First(); recording; recording = Next(recording)) {
-      if (strcmp(recording->FileName(), FileName) == 0)
-         return recording;
-      }
+  if (FileName) {
+     for (cRecording *recording = First(); recording; recording = Next(recording)) {
+         if (strcmp(recording->FileName(), FileName) == 0)
+            return recording;
+         }
+     }
   return NULL;
 }
 
@@ -1014,7 +1041,7 @@ int cRecordings::TotalFileSizeMB(void)
   int size = 0;
   LOCK_THREAD;
   for (cRecording *recording = First(); recording; recording = Next(recording)) {
-      if (recording->fileSizeMB > 0)
+      if (recording->fileSizeMB > 0 && IsOnVideoDirectoryFileSystem(recording->FileName()))
          size += recording->fileSizeMB;
       }
   return size;
@@ -1045,9 +1072,7 @@ cMark::~cMark()
 
 cString cMark::ToText(void)
 {
-  char *buffer;
-  asprintf(&buffer, "%s%s%s\n", *IndexToHMSF(position, true), comment ? " " : "", comment ? comment : "");
-  return cString(buffer, true);
+  return cString::sprintf("%s%s%s\n", *IndexToHMSF(position, true), comment ? " " : "", comment ? comment : "");
 }
 
 bool cMark::Parse(const char *s)
@@ -1136,11 +1161,9 @@ const char *cRecordingUserCommand::command = NULL;
 void cRecordingUserCommand::InvokeCommand(const char *State, const char *RecordingFileName)
 {
   if (command) {
-     char *cmd;
-     asprintf(&cmd, "%s %s \"%s\"", command, State, *strescape(RecordingFileName, "\"$"));
-     isyslog("executing '%s'", cmd);
+     cString cmd = cString::sprintf("%s %s \"%s\"", command, State, *strescape(RecordingFileName, "\\\"$"));
+     isyslog("executing '%s'", *cmd);
      SystemExec(cmd);
-     free(cmd);
      }
 }
 
@@ -1384,10 +1407,6 @@ bool cIndexFile::IsStillRecording()
 
 // --- cFileName -------------------------------------------------------------
 
-#include <errno.h>
-#include <unistd.h>
-#include "videodir.h"
-
 #define MAXFILESPERRECORDING 255
 #define RECORDFILESUFFIX    "/%03d.vdr"
 #define RECORDFILESUFFIXLEN 20 // some additional bytes for safety...
