@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: recording.h 1.56 2006/12/01 15:06:07 kls Exp $
+ * $Id: recording.h 1.59 2007/10/14 10:11:34 kls Exp $
  */
 
 #ifndef __RECORDING_H
@@ -42,6 +42,7 @@ class cRecordingInfo {
   friend class cRecording;
 private:
   tChannelID channelID;
+  char *channelName;
   const cEvent *event;
   cEvent *ownEvent;
   char *aux;
@@ -51,6 +52,7 @@ private:
 public:
   ~cRecordingInfo();
   tChannelID ChannelID(void) const { return channelID; }
+  const char *ChannelName(void) const { return channelName; }
   const char *Title(void) const { return event->Title(); }
   const char *ShortText(void) const { return event->ShortText(); }
   const char *Description(void) const { return event->Description(); }
@@ -70,6 +72,8 @@ private:
   mutable char *name;
   mutable int fileSizeMB;
   cRecordingInfo *info;
+  cRecording(const cRecording&); // can't copy cRecording
+  cRecording &operator=(const cRecording &); // can't assign cRecording
   static char *StripEpisodeName(char *s);
   char *SortName(void) const;
   int GetResume(void) const;
@@ -98,6 +102,10 @@ public:
   bool Remove(void);
        // Actually removes the file from the disk
        // Returns false in case of error
+  bool Undelete(void);
+       // Changes the file name so that it will be visible in the "Recordings" menu again and
+       // not processed by cRemoveDeletedRecordingsThread.
+       // Returns false in case of error
   };
 
 class cRecordings : public cList<cRecording>, public cThread {
