@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: remote.c 1.55 2006/12/02 11:12:42 kls Exp $
+ * $Id: remote.c 1.59 2008/02/23 14:14:46 kls Exp $
  */
 
 #include "remote.h"
@@ -31,6 +31,8 @@ cMutex cRemote::mutex;
 cCondVar cRemote::keyPressed;
 const char *cRemote::keyMacroPlugin = NULL;
 const char *cRemote::callPlugin = NULL;
+bool cRemote::enabled = true;
+time_t cRemote::lastActivity = 0;
 
 cRemote::cRemote(const char *Name)
 {
@@ -119,7 +121,7 @@ bool cRemote::PutMacro(eKeys Key)
 bool cRemote::Put(uint64_t Code, bool Repeat, bool Release)
 {
   char buffer[32];
-  snprintf(buffer, sizeof(buffer), "%016LX", Code);
+  snprintf(buffer, sizeof(buffer), "%016llX", Code);
   return Put(buffer, Repeat, Release);
 }
 
@@ -183,7 +185,8 @@ eKeys cRemote::Get(int WaitMs, char **UnknownCode)
             out = 0;
          if ((k & k_Repeat) != 0)
             repeatTimeout.Set(REPEATTIMEOUT);
-         return k;
+         TriggerLastActivity();
+         return enabled ? k : kNone;
          }
       else if (!WaitMs || !keyPressed.TimedWait(mutex, WaitMs) && repeatTimeout.TimedOut())
          return kNone;
@@ -195,6 +198,11 @@ eKeys cRemote::Get(int WaitMs, char **UnknownCode)
       }
 }
 
+void cRemote::TriggerLastActivity(void)
+{
+  lastActivity = time(NULL);
+}
+
 // --- cRemotes --------------------------------------------------------------
 
 cRemotes Remotes;
