@@ -4,7 +4,7 @@
  * See the main source file 'vdr.c' for copyright information and
  * how to reach the author.
  *
- * $Id: status.c 1.7 2005/01/09 11:51:04 kls Exp $
+ * $Id: status.c 1.10 2008/02/16 14:46:31 kls Exp $
  */
 
 #include "status.h"
@@ -23,22 +23,28 @@ cStatus::~cStatus()
   statusMonitors.Del(this, false);
 }
 
+void cStatus::MsgTimerChange(const cTimer *Timer, eTimerChange Change)
+{
+  for (cStatus *sm = statusMonitors.First(); sm; sm = statusMonitors.Next(sm))
+      sm->TimerChange(Timer, Change);
+}
+
 void cStatus::MsgChannelSwitch(const cDevice *Device, int ChannelNumber)
 {
   for (cStatus *sm = statusMonitors.First(); sm; sm = statusMonitors.Next(sm))
       sm->ChannelSwitch(Device, ChannelNumber);
 }
 
-void cStatus::MsgRecording(const cDevice *Device, const char *Name)
+void cStatus::MsgRecording(const cDevice *Device, const char *Name, const char *FileName, bool On)
 {
   for (cStatus *sm = statusMonitors.First(); sm; sm = statusMonitors.Next(sm))
-      sm->Recording(Device, Name);
+      sm->Recording(Device, Name, FileName, On);
 }
 
-void cStatus::MsgReplaying(const cControl *Control, const char *Name)
+void cStatus::MsgReplaying(const cControl *Control, const char *Name, const char *FileName, bool On)
 {
   for (cStatus *sm = statusMonitors.First(); sm; sm = statusMonitors.Next(sm))
-      sm->Replaying(Control, Name);
+      sm->Replaying(Control, Name, FileName, On);
 }
 
 void cStatus::MsgSetVolume(int Volume, bool Absolute)
@@ -59,6 +65,12 @@ void cStatus::MsgSetAudioChannel(int AudioChannel)
       sm->SetAudioChannel(AudioChannel);
 }
 
+void cStatus::MsgSetSubtitleTrack(int Index, const char * const *Tracks)
+{
+  for (cStatus *sm = statusMonitors.First(); sm; sm = statusMonitors.Next(sm))
+      sm->SetSubtitleTrack(Index, Tracks);
+}
+
 void cStatus::MsgOsdClear(void)
 {
   for (cStatus *sm = statusMonitors.First(); sm; sm = statusMonitors.Next(sm))
